DROP TABLE `grb_playstatus`;
DROP TABLE `grb_cds_resource`;
DROP TABLE `mt_metadata`;
DROP TABLE `mt_autoscan`;
DROP TABLE `mt_cds_object`;
DROP TABLE `mt_internal_setting`;
DROP TABLE `grb_config_value`;
DROP TABLE `grb_client`;
