DROP TABLE IF EXISTS "grb_playstatus";
DROP TABLE IF EXISTS "grb_cds_resource";
DROP TABLE IF EXISTS "mt_metadata";
DROP TABLE IF EXISTS "mt_autoscan";
DROP TABLE IF EXISTS "mt_cds_object";
DROP TABLE IF EXISTS "mt_internal_setting";
DROP TABLE IF EXISTS "grb_config_value";
DROP TABLE IF EXISTS "grb_client";

VACUUM;
