/* CSS - Cascading Style Sheet */
/* Palette color codes */
/* Palette URL: http://paletton.com/#uid=70z0u0kw0w0jvDaoOy4y4oODajv */

/* Feel free to copy&paste color codes to your application */


/* As hex codes */

.color-primary-0 { color: #FF7F00 }	/* Main Primary color */
.color-primary-1 { color: #FFB164 }
.color-primary-2 { color: #FF9B39 }
.color-primary-3 { color: #C66200 }
.color-primary-4 { color: #9B4D00 }

.color-secondary-1-0 { color: #FFB100 }	/* Main Secondary color (1) */
.color-secondary-1-1 { color: #FFD064 }
.color-secondary-1-2 { color: #FFC339 }
.color-secondary-1-3 { color: #C68900 }
.color-secondary-1-4 { color: #9B6C00 }

.color-secondary-2-0 { color: #1435AD }	/* Main Secondary color (2) */
.color-secondary-2-1 { color: #5A70C2 }
.color-secondary-2-2 { color: #3853B2 }
.color-secondary-2-3 { color: #0F2886 }
.color-secondary-2-4 { color: #091E69 }

.color-complement-0 { color: #04819E }	/* Main Complement color */
.color-complement-1 { color: #4BA2B7 }
.color-complement-2 { color: #298DA4 }
.color-complement-3 { color: #03647A }
.color-complement-4 { color: #024E60 }



/* As RGBa codes */

.rgba-primary-0 { color: rgba(255,127,  0,1) }	/* Main Primary color */
.rgba-primary-1 { color: rgba(255,177,100,1) }
.rgba-primary-2 { color: rgba(255,155, 57,1) }
.rgba-primary-3 { color: rgba(198, 98,  0,1) }
.rgba-primary-4 { color: rgba(155, 77,  0,1) }

.rgba-secondary-1-0 { color: rgba(255,177,  0,1) }	/* Main Secondary color (1) */
.rgba-secondary-1-1 { color: rgba(255,208,100,1) }
.rgba-secondary-1-2 { color: rgba(255,195, 57,1) }
.rgba-secondary-1-3 { color: rgba(198,137,  0,1) }
.rgba-secondary-1-4 { color: rgba(155,108,  0,1) }

.rgba-secondary-2-0 { color: rgba( 20, 53,173,1) }	/* Main Secondary color (2) */
.rgba-secondary-2-1 { color: rgba( 90,112,194,1) }
.rgba-secondary-2-2 { color: rgba( 56, 83,178,1) }
.rgba-secondary-2-3 { color: rgba( 15, 40,134,1) }
.rgba-secondary-2-4 { color: rgba(  9, 30,105,1) }

.rgba-complement-0 { color: rgba(  4,129,158,1) }	/* Main Complement color */
.rgba-complement-1 { color: rgba( 75,162,183,1) }
.rgba-complement-2 { color: rgba( 41,141,164,1) }
.rgba-complement-3 { color: rgba(  3,100,122,1) }
.rgba-complement-4 { color: rgba(  2, 78, 96,1) }



/* Generated by Paletton.com Â© 2002-2014 */
/* http://paletton.com */